package cn.iocoder.yudao.framework.common.util.date;

import cn.hutool.core.date.LocalDateTimeUtil;

import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.time.temporal.TemporalAdjusters;
import java.util.ArrayList;
import java.util.List;

/**
 * 时间工具类，用于 {@link java.time.LocalDateTime}
 */
public class LocalDateTimeUtils {

    /**
     * 空的 LocalDateTime 对象，主要用于 DB 唯一索引的默认值
     */
    public static LocalDateTime EMPTY = buildTime(1970, 1, 1);

    public static LocalDateTime addTime(Duration duration) {
        return LocalDateTime.now().plus(duration);
    }

    public static LocalDateTime minusTime(Duration duration) {
        return LocalDateTime.now().minus(duration);
    }

    public static boolean beforeNow(LocalDateTime date) {
        return date.isBefore(LocalDateTime.now());
    }

    public static boolean afterNow(LocalDateTime date) {
        return date.isAfter(LocalDateTime.now());
    }

    /**
     * 创建指定时间
     *
     * @param year  年
     * @param mouth 月
     * @param day   日
     * @return 指定时间
     */
    public static LocalDateTime buildTime(int year, int mouth, int day) {
        return LocalDateTime.of(year, mouth, day, 0, 0, 0);
    }

    public static LocalDateTime[] buildBetweenTime(int year1, int mouth1, int day1,
                                                   int year2, int mouth2, int day2) {
        return new LocalDateTime[]{buildTime(year1, mouth1, day1), buildTime(year2, mouth2, day2)};
    }

    /**
     * 判断当前时间是否在该时间范围内
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 是否
     */
    public static boolean isBetween(LocalDateTime startTime, LocalDateTime endTime) {
        if (startTime == null || endTime == null) {
            return false;
        }
        return LocalDateTimeUtil.isIn(LocalDateTime.now(), startTime, endTime);
    }

    /**
     * 判断当前时间是否在该时间范围内
     *
     * @param startTime 开始时间
     * @param endTime   结束时间
     * @return 是否
     */
    public static boolean isBetween(String startTime, String endTime) {
        if (startTime == null || endTime == null) {
            return false;
        }
        LocalDate nowDate = LocalDate.now();
        return LocalDateTimeUtil.isIn(LocalDateTime.now(),
                LocalDateTime.of(nowDate, LocalTime.parse(startTime)),
                LocalDateTime.of(nowDate, LocalTime.parse(endTime)));
    }

    /**
     * 判断时间段是否重叠
     *
     * @param startTime1 开始 time1
     * @param endTime1   结束 time1
     * @param startTime2 开始 time2
     * @param endTime2   结束 time2
     * @return 重叠：true 不重叠：false
     */
    public static boolean isOverlap(LocalTime startTime1, LocalTime endTime1, LocalTime startTime2, LocalTime endTime2) {
        LocalDate nowDate = LocalDate.now();
        return LocalDateTimeUtil.isOverlap(LocalDateTime.of(nowDate, startTime1), LocalDateTime.of(nowDate, endTime1),
                LocalDateTime.of(nowDate, startTime2), LocalDateTime.of(nowDate, endTime2));
    }

    /**
     * 获取指定日期所在的月份的开始时间
     * 例如：2023-09-30 00:00:00,000
     *
     * @param date 日期
     * @return 月份的开始时间
     */
    public static LocalDateTime beginOfMonth(LocalDateTime date) {
        return date.with(TemporalAdjusters.firstDayOfMonth()).with(LocalTime.MIN);
    }

    /**
     * 获取指定日期所在的月份的最后时间
     * 例如：2023-09-30 23:59:59,999
     *
     * @param date 日期
     * @return 月份的结束时间
     */
    public static LocalDateTime endOfMonth(LocalDateTime date) {
        return date.with(TemporalAdjusters.lastDayOfMonth()).with(LocalTime.MAX);
    }

    /**
     * 根据当前日期获取本季度最后一天
     *
     * @param dateTime
     * @return
     */
    public static LocalDateTime getQuarterEnd(LocalDateTime dateTime) {
        int month = dateTime.getMonthValue();
        int year = dateTime.getYear();

        LocalDateTime quarterEnd = LocalDateTime.now();

        if (month <= 3) {
            quarterEnd = LocalDateTime.of(year, 3, 31, 23, 59, 59);
        } else if (month <= 6) {
            quarterEnd = LocalDateTime.of(year, 6, 30, 23, 59, 59);
        } else if (month <= 9) {
            quarterEnd = LocalDateTime.of(year, 9, 30, 23, 59, 59);
        } else {
            quarterEnd = LocalDateTime.of(year, 12, 31, 23, 59, 59);
        }

        return quarterEnd.with(TemporalAdjusters.lastDayOfMonth());
    }

    /**
     * 获取至某一个时间每一个月的某一天
     *
     * @param beginTime
     * @param endTime
     * @param day
     */
    public static List<LocalDateTime> getTheDayOfEachMonthSoFar(LocalDateTime beginTime, LocalDateTime endTime, int day) {
        List<LocalDateTime> monthlyFirstDays = new ArrayList<>();
        beginTime = beginTime.withDayOfMonth(day);
        while (!beginTime.isAfter(endTime)) {
            // 获取当月的1号
            LocalDateTime firstDayOfMonth = beginTime.withDayOfMonth(day);
            // 添加到列表
            monthlyFirstDays.add(firstDayOfMonth);

            // 移动到下个月
            beginTime = beginTime.plusMonths(1);
        }
        return monthlyFirstDays;
    }


    /**
     * 获取至某一个时间每年的一月一号
     *
     * @param beginTime
     * @param endTime
     */
    public static List<LocalDateTime> getTheDayOfEachYeasSoFar(LocalDateTime beginTime, LocalDateTime endTime) {
        List<LocalDateTime> monthlyFirstDays = new ArrayList<>();
        while (!beginTime.isAfter(endTime)) {
            // 获取当月的1号
            LocalDateTime firstDayOfMonth = beginTime.withDayOfYear(1);
            // 添加到列表
            monthlyFirstDays.add(firstDayOfMonth);

            // 移动到下个月
            beginTime = beginTime.plusYears(1);
        }
        return monthlyFirstDays;
    }

    /**
     * 获取至某一个时间每年当前日期
     *
     * @param beginTime
     * @param endTime
     */
    public static List<LocalDateTime> getTheCurrentDateOfEachYearToACertainTime(LocalDateTime beginTime, LocalDateTime endTime) {
        List<LocalDateTime> monthlyFirstDays = new ArrayList<>();
        while (!beginTime.isAfter(endTime)) {
            // 添加到列表
            monthlyFirstDays.add(beginTime);
            // 移动到下个年
            beginTime = beginTime.plusYears(1);
        }
        return monthlyFirstDays;
    }

    /**
     * 判断两个时间范围是否相交 true 是 false 否
     */
    public static boolean intersects(LocalDateTime start, LocalDateTime end, LocalDateTime otherStart, LocalDateTime otherEnd) {
        // 判断两个时间范围是否相交
        return !end.isBefore(otherStart) && !otherEnd.isBefore(start);
    }

}
