package cn.iocoder.yudao.module.system.service.dept;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.util.ObjectUtil;
import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.enums.DeptTypeEnum;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.datapermission.core.annotation.DataPermission;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.system.api.dept.dto.DeptApiDTO;
import cn.iocoder.yudao.module.system.controller.admin.dept.vo.dept.DeptListReqVO;
import cn.iocoder.yudao.module.system.controller.admin.dept.vo.dept.DeptSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.dept.DeptDO;
import cn.iocoder.yudao.module.system.dal.mysql.dept.DeptMapper;
import cn.iocoder.yudao.module.system.dal.redis.RedisKeyConstants;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.google.common.annotations.VisibleForTesting;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cache.annotation.CacheEvict;
import org.springframework.cache.annotation.Cacheable;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.*;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.framework.common.util.collection.CollectionUtils.convertSet;
import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.TASK_OPERATE_FAIL_USER_NO_DEPT;
import static cn.iocoder.yudao.module.system.enums.ErrorCodeConstants.*;

/**
 * 部门 Service 实现类
 */
@Service
@Validated
@Slf4j
public class DeptServiceImpl implements DeptService {

    @Resource
    private DeptMapper deptMapper;

    @Override
    @CacheEvict(cacheNames = RedisKeyConstants.DEPT_CHILDREN_ID_LIST,
            allEntries = true) // allEntries 清空所有缓存，因为操作一个部门，涉及到多个缓存
    public Long createDept(DeptSaveReqVO createReqVO) {
        if (createReqVO.getParentId() == null) {
            createReqVO.setParentId(DeptDO.PARENT_ID_ROOT);
        }
        // 校验父部门的有效性
        validateParentDept(null, createReqVO.getParentId());
        // 校验部门名的唯一性
        validateDeptNameUnique(null, createReqVO.getParentId(), createReqVO.getName());

        // 插入部门
        DeptDO dept = BeanUtils.toBean(createReqVO, DeptDO.class);
        deptMapper.insert(dept);

        if ("0".equals(dept.getParentId().toString())) {

            dept.setLevel(0);
            dept.setFlag(dept.getId().toString());
        } else {

            //获取上级部门信息
            DeptDO patentDeptDO = getDept(dept.getParentId());

            //新建部门层级为 上级部门层级+1
            dept.setLevel(patentDeptDO.getLevel() + 1);
            //新建部门标识符为 上级部门标识符+本部门ID
            dept.setFlag(patentDeptDO.getFlag() + "-" + dept.getId().toString());
        }

        //更新部门 信息
        deptMapper.updateById(dept);
        return dept.getId();
    }

    @Override
    @CacheEvict(cacheNames = RedisKeyConstants.DEPT_CHILDREN_ID_LIST,
            allEntries = true) // allEntries 清空所有缓存，因为操作一个部门，涉及到多个缓存
    public void updateDept(DeptSaveReqVO updateReqVO) {
        if (updateReqVO.getParentId() == null) {
            updateReqVO.setParentId(DeptDO.PARENT_ID_ROOT);
        }
        // 校验自己存在
        validateDeptExists(updateReqVO.getId());
        // 校验父部门的有效性
        validateParentDept(updateReqVO.getId(), updateReqVO.getParentId());
        // 校验部门名的唯一性
        validateDeptNameUnique(updateReqVO.getId(), updateReqVO.getParentId(), updateReqVO.getName());

        //更新前部门 dept
        DeptDO oldDeptDO = getDept(updateReqVO.getId());

        // 更新部门
        DeptDO updateObj = BeanUtils.toBean(updateReqVO, DeptDO.class);

        if (updateObj.getLeaderUserId() == null) {
            updateObj.setLeaderUserId(0L);
        }

        // 如果变换上级部门
        if (!updateObj.getParentId().toString().equals(oldDeptDO.getParentId().toString())) {

            //获取更新后上级部门信息
            DeptDO patentDeptDO = getDept(updateObj.getParentId());
            //获取所有下级部门
            List<DeptDO> deptDOList = deptMapper.selectLikeFlag(oldDeptDO.getFlag());

            String oldFlag = oldDeptDO.getFlag();
            String newFlag = patentDeptDO.getFlag() + "-" + updateObj.getId().toString();

            //遍历下级部门设置部门层级和标识符
            for (DeptDO deptDO : deptDOList) {

                if (deptDO.getId().toString().equals(updateObj.getId().toString())) {
                    continue;
                }
                String flag = deptDO.getFlag();
                deptDO.setFlag(flag.replace(oldFlag, newFlag));
                deptDO.setLevel(deptDO.getFlag().split("-").length - 1);

                deptMapper.updateById(deptDO);
            }

            updateObj.setLevel(patentDeptDO.getLevel() + 1);
            updateObj.setFlag(newFlag);
        }

        deptMapper.updateById(updateObj);
    }

    @Override
    public void updateFactoryDept(DeptSaveReqVO updateReqVO) {

        // 校验自己存在
        validateFactoryDeptExists(updateReqVO.getFactoryId());

        DeptDO update = BeanUtils.toBean(updateReqVO, DeptDO.class);
        deptMapper.update(update, new LambdaQueryWrapperX<DeptDO>()
                .eq(DeptDO::getFactoryId, updateReqVO.getFactoryId()));
    }

    @Override
    @CacheEvict(cacheNames = RedisKeyConstants.DEPT_CHILDREN_ID_LIST,
            allEntries = true) // allEntries 清空所有缓存，因为操作一个部门，涉及到多个缓存
    public void deleteDept(Long id) {
        // 校验是否存在
        validateDeptExists(id);
        // 校验是否有子部门
        if (deptMapper.selectCountByParentId(id) > 0) {
            throw exception(DEPT_EXITS_CHILDREN);
        }
        // 删除部门
        deptMapper.deleteById(id);
    }

    @VisibleForTesting
    void validateDeptExists(Long id) {
        if (id == null) {
            return;
        }
        DeptDO dept = deptMapper.selectById(id);
        if (dept == null) {
            throw exception(DEPT_NOT_FOUND);
        }
    }

    @VisibleForTesting
    void validateFactoryDeptExists(Long factoryId) {
        if (factoryId == null) {
            return;
        }
        DeptDO dept = deptMapper.selectOne(DeptDO::getFactoryId, factoryId);
        if (dept == null) {
            throw exception(FACTORY_DEPT_NOT_FOUND);
        }
    }

    @VisibleForTesting
    void validateParentDept(Long id, Long parentId) {
        if (parentId == null || DeptDO.PARENT_ID_ROOT.equals(parentId)) {
            return;
        }
        // 1. 不能设置自己为父部门
        if (Objects.equals(id, parentId)) {
            throw exception(DEPT_PARENT_ERROR);
        }
        // 2. 父部门不存在
        DeptDO parentDept = deptMapper.selectById(parentId);
        if (parentDept == null) {
            throw exception(DEPT_PARENT_NOT_EXITS);
        }
        // 3. 递归校验父部门，如果父部门是自己的子部门，则报错，避免形成环路
        if (id == null) { // id 为空，说明新增，不需要考虑环路
            return;
        }
        for (int i = 0; i < Short.MAX_VALUE; i++) {
            // 3.1 校验环路
            parentId = parentDept.getParentId();
            if (Objects.equals(id, parentId)) {
                throw exception(DEPT_PARENT_IS_CHILD);
            }
            // 3.2 继续递归下一级父部门
            if (parentId == null || DeptDO.PARENT_ID_ROOT.equals(parentId)) {
                break;
            }
            parentDept = deptMapper.selectById(parentId);
            if (parentDept == null) {
                break;
            }
        }
    }

    @VisibleForTesting
    void validateDeptNameUnique(Long id, Long parentId, String name) {
        DeptDO dept = deptMapper.selectByParentIdAndName(parentId, name);
        if (dept == null) {
            return;
        }
        // 如果 id 为空，说明不用比较是否为相同 id 的部门
        if (id == null) {
            throw exception(DEPT_NAME_DUPLICATE);
        }
        if (ObjectUtil.notEqual(dept.getId(), id)) {
            throw exception(DEPT_NAME_DUPLICATE);
        }
    }

    @Override
    public DeptDO getDept(Long id) {
        return deptMapper.selectById(id);
    }

    @Override
    public List<DeptDO> getDeptList(Collection<Long> ids) {
        if (CollUtil.isEmpty(ids)) {
            return Collections.emptyList();
        }
        return deptMapper.selectBatchIds(ids);
    }

    @Override
    public List<DeptDO> getDeptList(DeptListReqVO reqVO) {
        List<DeptDO> list = deptMapper.selectList(reqVO);
        list.sort(Comparator.comparing(DeptDO::getSort));
        return list;
    }

    @Override
    public List<DeptDO> getChildDeptList(Long id) {
        List<DeptDO> children = new LinkedList<>();
        // 遍历每一层
        Collection<Long> parentIds = Collections.singleton(id);
        for (int i = 0; i < Short.MAX_VALUE; i++) { // 使用 Short.MAX_VALUE 避免 bug 场景下，存在死循环
            // 查询当前层，所有的子部门
            List<DeptDO> depts = deptMapper.selectListByParentId(parentIds);
            // 1. 如果没有子部门，则结束遍历
            if (CollUtil.isEmpty(depts)) {
                break;
            }
            // 2. 如果有子部门，继续遍历
            children.addAll(depts);
            parentIds = convertSet(depts, DeptDO::getId);
        }
        return children;
    }

    @Override
    @DataPermission(enable = false)
    @Cacheable(cacheNames = RedisKeyConstants.DEPT_CHILD_ID_LIST, key = "#id")
    public List<DeptDO> getChildDept(Long id) {

        return deptMapper.selectList(new LambdaQueryWrapperX<DeptDO>()
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .likeLeft(DeptDO::getFlag, "-" + id)
                .or()
                .likeRight(DeptDO::getFlag, id + "-")
                .or()
                .like(DeptDO::getFlag, "-" + id + "-"));
    }

    @Override
    @DataPermission(enable = false) // 禁用数据权限，避免建立不正确的缓存
    @Cacheable(cacheNames = RedisKeyConstants.DEPT_CHILDREN_ID_LIST, key = "#id")
    public Set<Long> getChildDeptIdListFromCache(Long id) {
        List<DeptDO> children = getChildDeptList(id);
        return convertSet(children, DeptDO::getId);
    }

    @Override
    public void validateDeptList(Collection<Long> ids) {
        if (CollUtil.isEmpty(ids)) {
            return;
        }
        // 获得科室信息
        Map<Long, DeptDO> deptMap = getDeptMap(ids);
        // 校验
        ids.forEach(id -> {
            DeptDO dept = deptMap.get(id);
            if (dept == null) {
                throw exception(DEPT_NOT_FOUND);
            }
            if (!CommonStatusEnum.ENABLE.getStatus().equals(dept.getStatus())) {
                throw exception(DEPT_NOT_ENABLE, dept.getName());
            }
        });
    }

    @Override
    public List<DeptDO> getDeptByLeaderId(Long leaderUserId) {

        return deptMapper.selectByLeaderId(leaderUserId);
    }

    @Override
    public List<DeptDO> getDeptTree() {

        return deptMapper.selectTree();
    }

    @Override
    @DataPermission(enable = false)
    public DeptDO getDeptByFactoryId(Long factoryId) {

        return deptMapper.selectOne(DeptDO::getFactoryId, factoryId);
    }

    @Override
    @DataPermission(enable = false)
    public List<DeptDO> getCompanyDept(Integer method) {

        return deptMapper.selectList(new LambdaQueryWrapperX<DeptDO>()
                .eq(method == 2, DeptDO::getVirtuallyStatus, 0)
                .eq(method == 3, DeptDO::getVirtuallyStatus, 1)
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .eq(DeptDO::getType, DeptTypeEnum.COMPANY.getValue())
                .or()
                .eq(DeptDO::getType, DeptTypeEnum.HEAD_COMPANY.getValue()));

    }

    @Override
    public List<DeptDO> getFactoryDeptList() {
        return this.getFactoryDeptList(null);
    }

    @Override
    public List<DeptDO> getFactoryDeptList(Long factoryId) {
        return deptMapper.selectList(new LambdaQueryWrapper<DeptDO>()
                .eq(factoryId != null, DeptDO::getFactoryId, factoryId)
                .isNotNull(DeptDO::getFactoryId)
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus()));
    }

    @Override
    public List<DeptDO> getDeptListByFactoryIds(Collection<Long> factoryIds) {
        return deptMapper.selectList(new LambdaQueryWrapper<DeptDO>()
                .in(DeptDO::getFactoryId, factoryIds));
    }

    @Override
    @DataPermission(enable = false)
    public DeptDO getUserCompanyDept(Long userId) {

        // 获取用户所在部门信息
        DeptDO deptDo = deptMapper.selectDeptByUserId(userId);

        if (deptDo == null) {
            throw exception(TASK_OPERATE_FAIL_USER_NO_DEPT);
        }

        // 根据所在部门信息获取 所在公司信息
        List<DeptDO> companyDeptList = deptMapper.selectCompany(deptDo.getFlag(), Arrays.asList(DeptTypeEnum.HEAD_COMPANY.getValue(), DeptTypeEnum.COMPANY.getValue()));

        if (CollectionUtil.isEmpty(companyDeptList)) {
            return new DeptDO();
        }

        return companyDeptList.get(0);
    }

    @Override
    public List<DeptDO> getAllList() {
        return deptMapper.selectList(new LambdaQueryWrapper<DeptDO>()
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus()));
    }

    @Override
    @DataPermission(enable = false)
    public List<DeptDO> getDeptListByType(String type) {
        return deptMapper.selectList(new LambdaQueryWrapper<DeptDO>()
                .eq(DeptDO::getType, type)
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus()));
    }

    @Override
    public List<DeptDO> getFactoryDept() {

        return deptMapper.selectList(new LambdaQueryWrapperX<DeptDO>()
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .isNotNull(DeptDO::getFactoryId));
    }

    @Override
    @DataPermission(enable = false)
    @Cacheable(cacheNames = RedisKeyConstants.COMPANY_DEPT, key = "#deptId")
    public DeptDO getCompanyByDept(Long deptId) {
        DeptDO deptDO = deptMapper.selectById(deptId);
        if (deptDO.getType().equals(DeptTypeEnum.COMPANY.getValue())) {
            return deptDO;
        }

        // 从缓存中获取所有非虚机构的公司信息
        List<DeptDO> companyDOs = this.getCompanyDept(2);
        return companyDOs.stream()
                .filter(company -> deptDO.getFlag().contains(company.getFlag()))
                .findFirst()
                .orElse(null);
    }

    @Override
    public List<DeptDO> getFactoryDeptByLeaderId(Long id) {
        return deptMapper.selectList(new LambdaQueryWrapperX<DeptDO>()
                .eq(DeptDO::getLeaderUserId, id)
                .eq(DeptDO::getStatus, CommonStatusEnum.ENABLE.getStatus())
                .isNotNull(DeptDO::getFactoryId));
    }

    @Override
    public List<DeptDO> getDeptList(DeptApiDTO dto) {
        List<DeptDO> list = deptMapper.selectList(new LambdaQueryWrapperX<DeptDO>()
                .likeIfPresent(DeptDO::getName, dto.getName())
                .eqIfPresent(DeptDO::getStatus, dto.getStatus()));
        list.sort(Comparator.comparing(DeptDO::getSort));
        return list;
    }
}
