package cn.iocoder.yudao.module.smartfactory.service.attendance;

import cn.hutool.core.collection.CollUtil;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.mybatis.core.query.LambdaQueryWrapperX;
import cn.iocoder.yudao.module.smartfactory.controller.admin.attendance.vo.StaffAttendanceRecordReqVO;
import cn.iocoder.yudao.module.smartfactory.controller.admin.attendance.vo.StaffAttendanceRecordSaveReqVO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.attendance.SfAttendanceReviewDO;
import cn.iocoder.yudao.module.smartfactory.dal.dataobject.attendance.StaffAttendanceRecordDO;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.attendance.SfAttendanceReviewMapper;
import cn.iocoder.yudao.module.smartfactory.dal.mysql.attendance.StaffAttendanceRecordMapper;
import cn.iocoder.yudao.module.smartfactory.service.staffsalary.StaffSalaryService;
import cn.iocoder.yudao.module.smartfactory.framework.job.staffSalary.StaffSalaryJob;
import cn.iocoder.yudao.module.smartfactory.service.staffsalary.StaffSalaryService;
import org.springframework.data.redis.connection.stream.ObjectRecord;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.smartfactory.enums.ErrorCodeConstants.STAFF_ATTENDANCE_RECORD_NOT_EXISTS;

/**
 * 厂区员工考勤记录 Service 实现类
 *
 * @author 符溶馨
 */
@Service
@Validated
public class StaffAttendanceRecordServiceImpl implements StaffAttendanceRecordService {

    @Resource
    private StaffAttendanceRecordMapper staffAttendanceRecordMapper;

    @Resource
    private StaffSalaryService staffSalaryService;

    @Resource
    private SfAttendanceReviewMapper attendanceReviewMapper;

    @Override
    public void createStaffAttendanceRecord(List<StaffAttendanceRecordSaveReqVO> createReqVO) {

        if (CollUtil.isEmpty(createReqVO)) {
            return;
        }
        // 新增前 全量清楚考勤记录
        staffAttendanceRecordMapper.deleteRecord(createReqVO.get(0));

        List<StaffAttendanceRecordDO> createDo = new ArrayList<>();
        for (StaffAttendanceRecordSaveReqVO vo : createReqVO) {

            List<StaffAttendanceRecordDO> staffAttendanceRecord = BeanUtils.toBean(vo.getRecords(), StaffAttendanceRecordDO.class);
            // 移除status为空得数据
            staffAttendanceRecord.removeIf(item -> item.getStatus() == null);

            staffAttendanceRecord.forEach(item -> {
                // 设置厂区
                item.setFactoryId(vo.getFactoryId());
                // 设置用户
                item.setStaffId(vo.getStaffId());
                // 设置打卡类型
                item.setType(vo.getType());
                // 设置考勤月份
                item.setMonth(vo.getMonth());
            });

            createDo.addAll(staffAttendanceRecord);
        }
        // 插入
        staffAttendanceRecordMapper.insertBatch(createDo);
    }

    @Override
    public void updateStaffAttendanceRecord(List<StaffAttendanceRecordSaveReqVO> updateReqVO) {

        // 更新
        List<StaffAttendanceRecordDO> updateObj = updateReqVO.stream()
                .flatMap(item -> BeanUtils.toBean(item.getRecords(), StaffAttendanceRecordDO.class).stream())
                .collect(Collectors.toList());

        staffAttendanceRecordMapper.updateBatch(updateObj);
    }

    @Override
    public void deleteStaffAttendanceRecord(Long id) {
        // 校验存在
        validateStaffAttendanceRecordExists(id);
        // 删除
        staffAttendanceRecordMapper.deleteById(id);
    }

    @Override
    public List<StaffAttendanceRecordDO> getRecord(Long staffId, String month, Integer type) {

        List<StaffAttendanceRecordDO> recordDOS = new ArrayList<>();
        // 如果考勤方式 是上月25至本月24号 时
        if (type == 1) {
            // 获取当前月份 上一个月得第一天
            LocalDate date = LocalDate.parse(month + "-01", DateTimeFormatter.ISO_LOCAL_DATE)
                    .minusMonths(1);
            // 获取上月25日日期
            LocalDate beginDate = date.withDayOfMonth(25);
            // 计算上个月的最后一天
            LocalDate lastMonthEnd = date.withDayOfMonth(date.lengthOfMonth()); // 设置为上个月的最后一天

            List<StaffAttendanceRecordDO> lastMonthRecord = staffAttendanceRecordMapper.selectList(new LambdaQueryWrapperX<StaffAttendanceRecordDO>()
                    .eq(StaffAttendanceRecordDO::getStaffId, staffId)
                    .betweenIfPresent(StaffAttendanceRecordDO::getPunchTime, new LocalDate[]{ beginDate, lastMonthEnd }));

            recordDOS.addAll(lastMonthRecord);
        }

        // 获取当前月份得考勤记录
        List<StaffAttendanceRecordDO> currentMonthRecord = staffAttendanceRecordMapper.selectList(new LambdaQueryWrapperX<StaffAttendanceRecordDO>()
                .eq(StaffAttendanceRecordDO::getStaffId, staffId)
                .like(StaffAttendanceRecordDO::getPunchTime, month));
        recordDOS.addAll(currentMonthRecord);

        return recordDOS;
    }

    private void validateStaffAttendanceRecordExists(Long id) {
        if (staffAttendanceRecordMapper.selectById(id) == null) {
            throw exception(STAFF_ATTENDANCE_RECORD_NOT_EXISTS);
        }
    }

    @Override
    public Integer getType(Long factoryId, String month) {
        List<StaffAttendanceRecordDO> recordDOS = staffAttendanceRecordMapper.selectList(new LambdaQueryWrapperX<StaffAttendanceRecordDO>()
                .eq(StaffAttendanceRecordDO::getFactoryId, factoryId)
                .eq(StaffAttendanceRecordDO::getMonth, month));

        if (CollUtil.isNotEmpty(recordDOS)) {
            return recordDOS.get(0).getType();
        }
        return null;
    }

    @Override
    public List<StaffAttendanceRecordDO> getStaffAttendanceRecordList(StaffAttendanceRecordReqVO reqVO) {

        return staffAttendanceRecordMapper.selectList(new LambdaQueryWrapperX<StaffAttendanceRecordDO>()
                .eqIfPresent(StaffAttendanceRecordDO::getStaffId, reqVO.getStaffId())
                .eqIfPresent(StaffAttendanceRecordDO::getFactoryId, reqVO.getFactoryId())
                .eqIfPresent(StaffAttendanceRecordDO::getMonth, reqVO.getMonth()));
    }

    @Override
    public List<StaffAttendanceRecordDO> getOtherRecord(StaffAttendanceRecordReqVO reqVO) {
        LocalDate[] date = null;
        if (reqVO.getType() == 1 && reqVO.getMonth() != null) {
            // 获取查询月份第一天
            LocalDate now = LocalDate.parse(reqVO.getMonth() + "-01", DateTimeFormatter.ISO_LOCAL_DATE);
            // 获取上月26日日期
            LocalDate beginDate = now.minusMonths(1).withDayOfMonth(26);
            // 获取查询月份得第25日日期
            LocalDate lastMonthEnd = now.withDayOfMonth(25);

            date = new LocalDate[]{ beginDate, lastMonthEnd };
        }
        return staffAttendanceRecordMapper.selectRecordList(reqVO, date);
    }

    @Override
    public List<StaffAttendanceRecordDO> getRecordListGroupByStaffId(String month) {

        return staffAttendanceRecordMapper.selectList(new LambdaQueryWrapperX<StaffAttendanceRecordDO>()
                .eq(StaffAttendanceRecordDO::getStatus, 1)
                .eq(StaffAttendanceRecordDO::getMonth, month));
    }

    @Override
    public void updateReview(Long factoryId, String month) {

        attendanceReviewMapper.insert(new SfAttendanceReviewDO()
                .setFactoryId(factoryId)
                .setMonth(month)
                .setStatus(1));

        List<StaffAttendanceRecordDO> createDo = staffAttendanceRecordMapper.selectList(new LambdaQueryWrapperX<StaffAttendanceRecordDO>()
                .eq(StaffAttendanceRecordDO::getFactoryId, factoryId)
                .eq(StaffAttendanceRecordDO::getMonth, month));
        // 审核考勤记录后， 自动计算员工薪资
        staffSalaryService.calculateSalary(createDo);
    }

    @Override
    public Integer getIsReview(Long factoryId, String month) {
        SfAttendanceReviewDO attendanceReviewDO = attendanceReviewMapper.selectOne(new LambdaQueryWrapperX<SfAttendanceReviewDO>()
                .eq(SfAttendanceReviewDO::getFactoryId, factoryId)
                .eq(SfAttendanceReviewDO::getMonth, month));
        return attendanceReviewDO != null ? attendanceReviewDO.getStatus() : 0;
    }
}